@@ -10,7 +10,7 @@ use namespace::autoclean;
 
 extends 'App::Sqitch::Command';
 
-our $VERSION = '0.953';
+our $VERSION = '0.960';
 
 has note => (
     is       => 'ro',
@@ -26,14 +26,15 @@ sub options {
 }
 
 sub execute {
-    my ( $self, $name ) = @_;
+    my ( $self, $name, $change ) = @_;
     my $sqitch = $self->sqitch;
     my $plan   = $sqitch->plan;
 
     if (defined $name) {
         my $tag = $plan->tag(
-            name => $name,
-            note => join "\n\n" => @{ $self->note },
+            name   => $name,
+            change => $change,
+            note   => join "\n\n" => @{ $self->note },
         );
 
         # Make sure we have a note.
